@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015-2017, Intel Corporation
+ * Copyright (c) 2015-2018, Intel Corporation
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -26,6 +26,9 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 
+#ifdef _WIN32
+#define PCRE_STATIC
+#endif
 #include "config.h"
 
 #include "common.h"
@@ -100,7 +103,8 @@ int pcreCallOut(pcre_callout_block *block) {
 
 static
 bool decodeExprPcre(string &expr, unsigned *flags, bool *highlander,
-                    bool *prefilter, bool *som, hs_expr_ext *ext) {
+                    bool *prefilter, bool *som, bool *combination,
+                    bool *quiet, hs_expr_ext *ext) {
     string regex;
     unsigned int hs_flags = 0;
     if (!readExpression(expr, regex, &hs_flags, ext)) {
@@ -109,7 +113,8 @@ bool decodeExprPcre(string &expr, unsigned *flags, bool *highlander,
 
     expr.swap(regex);
 
-    if (!getPcreFlags(hs_flags, flags, highlander, prefilter, som)) {
+    if (!getPcreFlags(hs_flags, flags, highlander, prefilter, som,
+                      combination, quiet)) {
         return false;
     }
 
@@ -185,6 +190,14 @@ string pcreErrStr(int err) {
     }
 }
 
+/* that is, a mode provided by native hyperscan */
+static
+bool isStandardMode(unsigned int mode) {
+    return mode == MODE_BLOCK
+        || mode == MODE_STREAMING
+        || mode == MODE_VECTORED;
+}
+
 GroundTruth::GroundTruth(ostream &os, const ExpressionMap &expr,
                          unsigned long int limit,
                          unsigned long int limit_recursion)
@@ -192,8 +205,10 @@ GroundTruth::GroundTruth(ostream &os, const ExpressionMap &expr,
       matchLimitRecursion(limit_recursion) {}
 
 void GroundTruth::global_prep() {
-    // We're using pcre callouts
-    pcre_callout = &pcreCallOut;
+    if (isStandardMode(colliderMode)) {
+        // We're using pcre callouts
+        pcre_callout = &pcreCallOut;
+    }
 }
 
 static
@@ -221,6 +236,8 @@ GroundTruth::compile(unsigned id, bool no_callouts) {
     bool highlander = false;
     bool prefilter = false;
     bool som = false;
+    bool combination = false;
+    bool quiet = false;
 
     // we can still match approximate matching patterns with PCRE if edit
     // distance 0 is requested
@@ -238,7 +255,8 @@ GroundTruth::compile(unsigned id, bool no_callouts) {
     hs_expr_ext ext;
 
     // Decode the flags
-    if (!decodeExprPcre(re, &flags, &highlander, &prefilter, &som, &ext)) {
+    if (!decodeExprPcre(re, &flags, &highlander, &prefilter, &som,
+                        &combination, &quiet, &ext)) {
         throw PcreCompileFailure("Unable to decode flags.");
     }
 
@@ -257,11 +275,17 @@ GroundTruth::compile(unsigned id, bool no_callouts) {
         throw PcreCompileFailure("Unsupported extended flags.");
     }
 
+    // Hybrid mode implies SOM.
+    if (colliderMode == MODE_HYBRID) {
+        assert(!use_NFA);
+        som = true;
+    }
+
     // SOM flags might be set globally.
     som |= !!somFlags;
 
     // For traditional Hyperscan, add global callout to pattern.
-    if (!no_callouts) {
+    if (!combination && !no_callouts && isStandardMode(colliderMode)) {
         addCallout(re);
     }
 
@@ -275,12 +299,22 @@ GroundTruth::compile(unsigned id, bool no_callouts) {
     compiled->highlander = highlander;
     compiled->prefilter = prefilter;
     compiled->som = som;
+    compiled->combination = combination;
+    compiled->quiet = quiet;
     compiled->min_offset = ext.min_offset;
     compiled->max_offset = ext.max_offset;
     compiled->min_length = ext.min_length;
     compiled->expression = i->second; // original PCRE
     flags |= PCRE_NO_AUTO_POSSESS;
 
+    if (compiled->combination) {
+        compiled->pl.parseLogicalCombination(id, re.c_str(), ~0U, 0, ~0ULL);
+        compiled->pl.logicalKeyRenumber();
+        compiled->report = id;
+        return compiled;
+    }
+
+
     compiled->bytecode =
         pcre_compile2(re.c_str(), flags, &errcode, &errptr, &errloc, nullptr);
 
@@ -388,6 +422,79 @@ int scanBasic(const CompiledPcre &compiled, const string &buffer,
     return ret;
 }
 
+static
+bool isUtf8(const CompiledPcre &compiled) {
+    unsigned long int options = 0;
+    pcre_fullinfo(compiled.bytecode, NULL, PCRE_INFO_OPTIONS, &options);
+    return options & PCRE_UTF8;
+}
+
+static
+CaptureVec makeCaptureVec(const vector<int> &ovector, int ret) {
+    assert(ret > 0);
+
+    CaptureVec cap;
+
+    if (no_groups) {
+        return cap; // No group info requested.
+    }
+
+    cap.reserve(ret * 2);
+    for (int i = 0; i < ret * 2; i += 2) {
+        int from = ovector[i], to = ovector[i + 1];
+        cap.push_back(make_pair(from, to));
+    }
+    return cap;
+}
+
+static
+int scanHybrid(const CompiledPcre &compiled, const string &buffer,
+               const pcre_extra &extra, vector<int> &ovector,
+               ResultSet &rs, ostream &out) {
+    int len = (int)buffer.length();
+    int startoffset = 0;
+    bool utf8 = isUtf8(compiled);
+
+    int flags = 0;
+    int ret;
+    do {
+        ret = pcre_exec(compiled.bytecode, &extra, buffer.c_str(), len,
+                        startoffset, flags, &ovector[0], ovector.size());
+
+        if (ret <= PCRE_ERROR_NOMATCH) {
+            return ret;
+        }
+
+        int from = ovector.at(0);
+        int to = ovector.at(1);
+        rs.addMatch(from, to, makeCaptureVec(ovector, ret));
+
+        if (echo_matches) {
+            out << "PCRE Match @ (" << from << "," << to << ")" << endl;
+        }
+
+        // If we only wanted a single match, we're done.
+        if (compiled.highlander) break;
+
+        // Next scan starts at the first codepoint after the match. It's
+        // possible that we have a vacuous match, in which case we must step
+        // past it to ensure that we always progress.
+        if (from != to) {
+            startoffset = to;
+        } else if (utf8) {
+            startoffset = to + 1;
+            while (startoffset < len
+                   && ((buffer[startoffset] & 0xc0) == UTF_CONT_BYTE_HEADER)) {
+                ++startoffset;
+            }
+        } else {
+            startoffset = to + 1;
+        }
+    } while (startoffset <= len);
+
+    return ret;
+}
+
 static
 int scanOffset(const CompiledPcre &compiled, const string &buffer,
                const pcre_extra &extra, vector<int> &ovector,
@@ -424,22 +531,117 @@ int scanOffset(const CompiledPcre &compiled, const string &buffer,
     return ret;
 }
 
+/** \brief Returns 1 if compliant to all logical combinations. */
+static
+char isLogicalCombination(vector<char> &lv, const vector<LogicalOp> &comb,
+                          size_t lkeyCount, unsigned start, unsigned result) {
+    assert(start <= result);
+    for (unsigned i = start; i <= result; i++) {
+        const LogicalOp &op = comb[i - lkeyCount];
+        assert(i == op.id);
+        switch (op.op) {
+        case LOGICAL_OP_NOT:
+            lv[op.id] = !lv[op.ro];
+            break;
+        case LOGICAL_OP_AND:
+            lv[op.id] = lv[op.lo] & lv[op.ro]; // &&
+            break;
+        case LOGICAL_OP_OR:
+            lv[op.id] = lv[op.lo] | lv[op.ro]; // ||
+            break;
+        default:
+            assert(0);
+            break;
+        }
+    }
+    return lv[result];
+}
+
 bool GroundTruth::run(unsigned, const CompiledPcre &compiled,
                       const string &buffer, ResultSet &rs, string &error) {
+    if (compiled.quiet) {
+        return true;
+    }
+
+    if (compiled.combination) {
+        // Compile and run sub-expressions, store match results.
+        map<unsigned long long, set<MatchResult>> offset_to_matches;
+        map<unsigned long long, set<unsigned>> offset_to_lkeys;
+        set<unsigned> sub_exps;
+        const auto &m_lkey = compiled.pl.getLkeyMap();
+        for (const auto &it_lkey : m_lkey) {
+            if (sub_exps.find(it_lkey.first) == sub_exps.end()) {
+                sub_exps.emplace(it_lkey.first);
+                ResultSet sub_rs(RESULT_FROM_PCRE);
+                shared_ptr<CompiledPcre> sub_pcre;
+                try {
+                    sub_pcre = compile(it_lkey.first);
+                }
+                catch (const SoftPcreCompileFailure &err) {
+                    return false;
+                }
+                catch (const PcreCompileFailure &err) {
+                    return false;
+                }
+                sub_pcre->quiet = false; // force not quiet in sub-exp.
+                if (!run(it_lkey.first, *sub_pcre, buffer, sub_rs, error)) {
+                    rs.clear();
+                    return false;
+                }
+                for (const auto &it_mr : sub_rs.matches) {
+                    offset_to_matches[it_mr.to].emplace(it_mr);
+                    offset_to_lkeys[it_mr.to].emplace(it_lkey.second);
+                    if (sub_pcre->highlander) {
+                        break;
+                    }
+                }
+            }
+        }
+        // Calculate rs for combination expression.
+        vector<char> lv;
+        const auto &comb = compiled.pl.getLogicalTree();
+        lv.resize(m_lkey.size() + comb.size());
+        const auto &li = compiled.pl.getCombInfoById(compiled.report);
+        for (const auto &it : offset_to_lkeys) {
+            for (auto report : it.second) {
+                lv[report] = 1;
+            }
+            if (isLogicalCombination(lv, comb, m_lkey.size(),
+                                     li.start, li.result)) {
+                for (const auto &mr : offset_to_matches.at(it.first)) {
+                    if ((mr.to >= compiled.min_offset) &&
+                        (mr.to <= compiled.max_offset)) {
+                        rs.addMatch(mr.from, mr.to);
+                    }
+                }
+            }
+        }
+        return true;
+    }
+
     CalloutContext ctx(out);
 
     pcre_extra extra;
     extra.flags = 0;
 
-    // Switch on callouts.
-    extra.flags |= PCRE_EXTRA_CALLOUT_DATA;
-    extra.callout_data = &ctx;
+    // If running in traditional HyperScan mode, switch on callouts.
+    bool usingCallouts = isStandardMode(colliderMode);
+    if (usingCallouts) {
+        // Switch on callouts.
+        extra.flags |= PCRE_EXTRA_CALLOUT_DATA;
+        extra.callout_data = &ctx;
+    }
 
     // Set the match_limit (in order to bound execution time on very complex
     // patterns)
     extra.flags |= (PCRE_EXTRA_MATCH_LIMIT | PCRE_EXTRA_MATCH_LIMIT_RECURSION);
-    extra.match_limit = matchLimit;
-    extra.match_limit_recursion = matchLimitRecursion;
+    if (colliderMode == MODE_HYBRID) {
+        extra.match_limit = 10000000;
+        extra.match_limit_recursion = 1500;
+    } else {
+        extra.match_limit = matchLimit;
+        extra.match_limit_recursion = matchLimitRecursion;
+    }
 
 #ifdef PCRE_NO_START_OPTIMIZE
     // Switch off optimizations that may result in callouts not occurring.
@@ -452,6 +654,7 @@ bool GroundTruth::run(unsigned, const CompiledPcre &compiled,
     ovector.resize(ovecsize);
 
     int ret;
+    bool hybrid = false;
     switch (colliderMode) {
     case MODE_BLOCK:
     case MODE_STREAMING:
@@ -462,6 +665,10 @@ bool GroundTruth::run(unsigned, const CompiledPcre &compiled,
             ret = scanBasic(compiled, buffer, extra, ovector, ctx);
         }
         break;
+    case MODE_HYBRID:
+        ret = scanHybrid(compiled, buffer, extra, ovector, rs, out);
+        hybrid = true;
+        break;
     default:
         assert(0);
         ret = PCRE_ERROR_NULL;
@@ -494,7 +701,7 @@ bool GroundTruth::run(unsigned, const CompiledPcre &compiled,
         return true;
     }
 
-    if (compiled.som) {
+    if (compiled.som && !hybrid) {
         filterLeftmostSom(rs);
     }
 
